import json
import sys
import yaml

from .common import get_host_info


class Result(object):
    """
    Result collection class.
    """
    def __init__(self, name=None, testsuite=None, testcase=None, runlist=None):
        self.results = []
        self.status = 'PASS'
        self.name = name
        self.testsuite = testsuite
        self.testcase = testcase
        self.runlist = runlist

        self.errors = 0
        self.fetch_errors = 0
        self.failures = 0
        self.passes = 0

    def add_result(self, result):
        """
        Add a result to the object.

        Note: 'result' is expected to be a dictionary like this::

            {
                'command': '',
                'returncode': 0,
                'stdout': '',
                'stderr': '',
                'start_time': '',
                'time_delta': '',
            }
        """
        if result is None:
            return

        if self.testsuite is not None:
            result['testsuite'] = self.testsuite

        if self.testcase is not None:
            result['testcase'] = self.testcase

        self.results.append(result)

        if result['returncode'] == 1 and self.status != 'ERROR':
            self.status = 'FAIL'

        if result['returncode'] not in [0, 1]:
            self.status = 'ERROR'

    def result(self, verbose=False):
        """
        Output a text based result.
        """
        status = self.status
        sep = '-' * 70

        print sep

        if self.name:
            print self.name
            print sep

        for result in self.results:
            print "command: %s" % result['command']
            print "returned: %d" % result['returncode']
            print "started: %s" % result['start_time']
            print "runtime: %s" % result['time_delta']

            if self.testsuite is not None:
                print "testsuite: %s" % self.testsuite

            if self.testcase is not None:
                print "testcase: %s" % self.testcase

            if self.runlist is not None:
                print "runlist: %s" % self.runlist

            if self.status != 'PASS' or verbose and result['stdout'] != '':
                print "stdout: "
                print result['stdout']

            if self.status != 'PASS' or verbose and result['stderr'] != '':
                print "stderr: "
                print result['stderr']

        print

        host_info = get_host_info()

        print 'runlist: {}'.format(self.runlist)
        print 'errors: {}'.format(self.errors)
        print 'failures: {}'.format(self.failures)
        print 'fetch_errors: {}'.format(self.fetch_errors)
        print 'passes: {}'.format(self.passes)
        print 'uname: {}'.format(" ".join(list(host_info['uname'])))
        print 'media-info: {}'.format(host_info['media-info'])
        self.clear_results()

        return status

    def clear_results(self):
        # reset the list of results so this object can be reused.
        self.results = []

        # reset the status, this should be safe since the only places that
        # should be calling clear_results() is testcase.run(), testsuite.run(),
        # and runner after attempting to fetch the testsuite.
        self.status = 'PASS'

    def count_results(self):
        return len(self.results)


class ResultYAML(Result):
    def result(self, verbose=False):

        if self.results:
            host_info = get_host_info()
            data = {
                    'runlist': self.runlist,
                    'commands': self.results,
                    'errors': self.errors,
                    'failures': self.failures,
                    'fetch_errors': self.fetch_errors,
                    'passes': self.passes,
                    'uname': list(host_info['uname']),
                    'media-info': host_info['media-info'],
                    }
            yaml.dump(data, sys.stdout, explicit_start='---',
                      default_flow_style=False)

        status = self.status
        self.clear_results()

        return status


class ResultJSON(Result):
    def result(self, verbose=False):

        if self.results:
            host_info = get_host_info()
            data = {
                    'runlist': self.runlist,
                    'commands': self.results,
                    'errors': self.errors,
                    'failures': self.failures,
                    'fetch_errors': self.fetch_errors,
                    'passes': self.passes,
                    'uname': list(host_info['uname']),
                    'media-info': host_info['media-info'],
                    }
            json.dump(data, sys.stdout, indent=4)

        status = self.status
        self.clear_results()

        return status
